@@ -38,16 +38,16 @@ static size_t popupFontSize = 200;
 class SetSelector : public Lum::Popup
 {
 private:
-  Lum::Model::ActionRef models[10]; // Values 0..9, where 0 means empty
+  Lum::Model::ActionRef models[Sudoku::dimension+1]; // Values 0..9, where 0 means empty
   bool                  selected;
   size_t                value;
 
 public:
   SetSelector()
   : selected(false),
-    value(10)
+    value(Sudoku::dimension+1)
   {
-    for (size_t i=0; i<=9; i++) {
+    for (size_t i=0; i<=Sudoku::dimension; i++) {
       models[i]=new Lum::Model::Action();
       Observe(models[i]);
     }
@@ -64,7 +64,7 @@ public:
     array->SetFlex(true,true);
     array->SetHorizontalCount(3);
 
-    for (size_t i=1; i<=9; i++) {
+    for (size_t i=1; i<=Sudoku::dimension; i++) {
       Lum::Text   *text;
 
       text=new Lum::Text();
@@ -86,7 +86,7 @@ public:
 
   void Resync(Lum::Base::Model *model, const Lum::Base::ResyncMsg& msg)
   {
-    for (size_t i=0; i<=9; ++i) {
+    for (size_t i=0; i<=Sudoku::dimension; ++i) {
       if (model==models[i] && models[i]->IsFinished()) {
         selected=true;
         value=i;
@@ -112,7 +112,7 @@ public:
 class MarkSelector : public Lum::Popup
 {
 private:
-  Lum::Model::BooleanRef models[9];
+  Lum::Model::BooleanRef models[Sudoku::dimension];
   size_t                 value;
 
 public:
@@ -189,12 +189,12 @@ GameArea::GameArea()
    markFont(Lum::OS::display->GetFontByPixel(Lum::OS::Display::fontTypeProportional,7)),
    font(Lum::OS::display->GetFont(Lum::OS::Display::fontScaleCaption1)),
    presetColor(0.5,0.5,0.5,Lum::OS::display->GetColor(Lum::OS::Display::whiteColor)),
-   focusBackgroundColor(0.95,0.95,0.95,Lum::OS::display->GetColor(Lum::OS::Display::fillColor)),
-   markBackgroundColor(0.95,0.95,0.95,Lum::OS::display->GetColor(Lum::OS::Display::whiteColor)),
+   focusBackgroundColor(0.8,0.8,0.8,Lum::OS::display->GetColor(Lum::OS::Display::fillColor)),
    errorColor(1.0,0,0,Lum::OS::display->GetColor(Lum::OS::Display::fillColor)),
    correctColor(0,0.5,0,Lum::OS::display->GetColor(Lum::OS::Display::blackColor)),
    boxSize(0),
    focusedCell(0),
+   focusChangedAction(new Lum::Model::Action()),
    finished(new Lum::Model::Boolean())
 {
   SetCanFocus(true);
@@ -219,8 +219,10 @@ void GameArea::SetModel(const Sudoku &riddle, const Sudoku &solution)
   }
 
   finished->Set(game==solution);
+  focusedCell=0;
 
   Redraw();
+  focusChangedAction->Trigger();
 }
 
 void GameArea::SetGame(const Game& game)
@@ -242,6 +244,11 @@ void GameArea::SetGame(const Game& game)
   }
 
   finished->Set(this->game==solution);
+
+  focusedCell=0;
+  Redraw();
+
+  focusChangedAction->Trigger();
 }
 
 void GameArea::GetGame(Game& game) const
@@ -269,12 +276,22 @@ void GameArea::GetGame(Sudoku& game) const
   game=this->game;
 }
 
+void GameArea::GetRiddle(Sudoku& game) const
+{
+  game=this->riddle;
+}
+
+void GameArea::GetSolution(Sudoku& game) const
+{
+  game=this->solution;
+}
+
 void GameArea::CalcSize()
 {
   size_t markBoxSize=0;
   size_t minBoxSize=0;
 
-  for (size_t i=1; i<=9; i++) {
+  for (size_t i=1; i<=Sudoku::dimension; i++) {
     Lum::OS::FontExtent extent;
 
     minFont->StringExtent(Lum::Base::NumberToWString(i),extent,Lum::OS::Font::bold);
@@ -285,7 +302,7 @@ void GameArea::CalcSize()
 
   minBoxSize+=2*Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder);
 
-  for (size_t i=1; i<=9; i++) {
+  for (size_t i=1; i<=Sudoku::dimension; i++) {
     Lum::OS::FontExtent extent;
 
     markFont->StringExtent(Lum::Base::NumberToWString(i),extent,Lum::OS::Font::normal);
@@ -335,7 +352,7 @@ Lum::OS::Font* GameArea::FindFont(size_t size)
     tmpFont=Lum::OS::display->GetFontByPixel(Lum::OS::Display::fontTypeProportional,fontSize);
 
     if (tmpFont.Valid()) {
-      for (size_t i=1; i<=9; i++) {
+      for (size_t i=1; i<=Sudoku::dimension; i++) {
         Lum::OS::FontExtent extent;
 
         tmpFont->StringExtent(Lum::Base::NumberToWString(i),extent,Lum::OS::Font::bold);
@@ -364,7 +381,7 @@ void GameArea::Draw(int x, int y, size_t w, size_t h)
 {
   Object::Draw(x,y,w,h);
 
-  if (!Intersect(x,y,w,h)) {
+  if (!OIntersect(x,y,w,h)) {
     return;
   }
 
@@ -415,7 +432,8 @@ void GameArea::Draw(int x, int y, size_t w, size_t h)
   for (size_t i=0; i<Sudoku::dimension*Sudoku::dimension; i++) {
 
     // Draw background
-    if (HasFocus() && !finished->Get() && i==focusedCell) {
+    if ((HasFocus() || Lum::OS::display->GetTheme()->RequestFingerFriendlyControls())
+        && !finished->Get() && i==focusedCell) {
       draw->PushForeground(focusBackgroundColor);
     }
     else {
@@ -515,9 +533,7 @@ bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
   }
 
   if ((event.type==Lum::OS::MouseEvent::down && PointIsIn(event) &&
-       event.button==Lum::OS::MouseEvent::button1) ||
-      (event.type==Lum::OS::MouseEvent::move && PointIsIn(event) &&
-       event.IsGrabed() && event.qualifier==Lum::OS::qualifierButton1)) {
+       event.button==Lum::OS::MouseEvent::button1)) {
     size_t tmp=0;
 
     while (tmp<Sudoku::dimension*Sudoku::dimension) {
@@ -531,34 +547,21 @@ bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
       tmp++;
     }
 
-    if (tmp<Sudoku::dimension*Sudoku::dimension && tmp!=focusedCell) {
-      focusedCell=tmp;
-      Redraw();
-    }
-
-    return true;
-  }
-  else if (event.type==Lum::OS::MouseEvent::up && PointIsIn(event) &&
-           event.IsGrabed() && event.button==Lum::OS::MouseEvent::button1) {
-
-    size_t tmp=0;
-
-    while (tmp<Sudoku::dimension*Sudoku::dimension) {
-      if (event.x>=this->x+GetCellOffset(tmp%Sudoku::dimension) &&
-          event.x<this->x+GetCellOffset(tmp%Sudoku::dimension)+(int)boxSize &&
-          event.y>=this->y+GetCellOffset(tmp/Sudoku::dimension) &&
-          event.y<this->y+GetCellOffset(tmp/Sudoku::dimension)+(int)boxSize) {
-        break;
-      }
-
-      tmp++;
-    }
-
-    if (tmp<Sudoku::dimension*Sudoku::dimension && riddle.Get(tmp)==Sudoku::empty) {
-
+    if (tmp<Sudoku::dimension*Sudoku::dimension) {
       if (tmp!=focusedCell) {
         focusedCell=tmp;
         Redraw();
+        focusChangedAction->Trigger();
+      }
+
+      if (riddle.Get(focusedCell)!=Sudoku::empty) {
+        return true;
+      }
+
+      if (Lum::OS::display->GetTheme()->RequestFingerFriendlyControls()) {
+        // No popup dialog, if we need to be finger friendly!
+        // We will make use of an external keypad instead in this case.
+        return true;
       }
 
       if (event.x-(this->x+GetCellOffset(tmp%Sudoku::dimension))<=(int)boxSize/2 &&
@@ -567,7 +570,8 @@ bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
 
         selector=new MarkSelector(marks[focusedCell]);
         selector->SetParent(GetWindow());
-        selector->SetPosition(GetWindow()->GetX()+event.x,GetWindow()->GetY()+event.y);
+        selector->SetPosition(GetWindow()->GetX()+event.x+5,GetWindow()->GetY()+event.y+5);
+        selector->GetWindow()->SetOpacity(0.8);
 
         if (selector->Open()) {
           selector->EventLoop();
@@ -588,7 +592,8 @@ bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
 
         selector=new SetSelector();
         selector->SetParent(GetWindow());
-        selector->SetPosition(GetWindow()->GetX()+event.x,GetWindow()->GetY()+event.y);
+        selector->SetPosition(GetWindow()->GetX()+event.x+5,GetWindow()->GetY()+event.y+5);
+        selector->GetWindow()->SetOpacity(0.8);
         selector->Open();
         selector->EventLoop();
         selector->Close();
@@ -602,7 +607,11 @@ bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
 
         delete selector;
       }
+
     }
+
+
+    return true;
   }
 
   return false;
@@ -622,24 +631,28 @@ bool GameArea::HandleKeyEvent(const Lum::OS::KeyEvent& event)
       if (focusedCell%Sudoku::dimension>0) {
         focusedCell--;
         Redraw();
+        focusChangedAction->Trigger();
       }
       return true;
     case Lum::OS::keyRight:
       if (focusedCell%Sudoku::dimension<Sudoku::dimension-1) {
         focusedCell++;
         Redraw();
+        focusChangedAction->Trigger();
       }
       return true;
     case Lum::OS::keyUp:
       if (focusedCell/Sudoku::dimension>0) {
         focusedCell-=Sudoku::dimension;
         Redraw();
+        focusChangedAction->Trigger();
       }
       return true;
     case Lum::OS::keyDown:
       if (focusedCell/Sudoku::dimension<Sudoku::dimension-1) {
         focusedCell+=Sudoku::dimension;
         Redraw();
+        focusChangedAction->Trigger();
       }
       return true;
     case Lum::OS::keySpace:
@@ -648,15 +661,19 @@ bool GameArea::HandleKeyEvent(const Lum::OS::KeyEvent& event)
       if (riddle.Get(focusedCell)==Sudoku::empty) {
         game.Set(focusedCell,Sudoku::empty);
         Redraw();
+        focusChangedAction->Trigger();
       }
       return true;
     default:
       if (Lum::Base::WStringToNumber(event.text,value)) {
-        if (value>=1 && value<=Sudoku::dimension) {
+        if (value>=0 && value<=Sudoku::dimension) {
           if (riddle.Get(focusedCell)==Sudoku::empty) {
             game.Set(focusedCell,value);
             finished->Set(game==solution);
             Redraw();
+            // Focus did not changed, but we simulate it to get the value change
+            // notified
+            focusChangedAction->Trigger();
           }
         }
 
@@ -669,11 +686,61 @@ bool GameArea::HandleKeyEvent(const Lum::OS::KeyEvent& event)
   return false;
 }
 
+Lum::Model::Action* GameArea::GetFocusChangedAction() const
+{
+  return focusChangedAction.Get();
+}
+
 Lum::Model::Boolean* GameArea::GetFinishedModel() const
 {
   return finished.Get();
 }
 
+bool GameArea::IsFocusedCellEditable() const
+{
+  return !finished->Get() && riddle.Get(focusedCell)==Sudoku::empty;
+}
+
+size_t GameArea::GetFocusedCellValue() const
+{
+  if (game.Get(focusedCell)!=Sudoku::empty) {
+    return 1 << (game.Get(focusedCell)-1);
+  }
+  else {
+    return marks[focusedCell];
+  }
+}
+
+void GameArea::SetFocusedCellValue(size_t value)
+{
+  assert(!GetFinishedModel()->Get());
+  assert(IsFocusedCellEditable());
+
+  size_t count=0;
+
+  for (size_t i=0; i<Sudoku::dimension; i++) {
+    if (value & (1 << i)) {
+      count++;
+    }
+  }
+
+  if (count==1) {
+    for (size_t i=0; i<Sudoku::dimension; i++) {
+      if (value & (1 << i)) {
+        game.Set(focusedCell,i+1);
+        marks[focusedCell]=0;
+        finished->Set(game==solution);
+        Redraw();
+      }
+    }
+  }
+  else {
+    game.Set(focusedCell,Sudoku::empty);
+    marks[focusedCell]=value;
+    Redraw();
+  }
+}
+
 void GameArea::Solve()
 {
   finished->Set(true);
