@@ -22,6 +22,7 @@
 
 #include <Lum/Object.h>
 
+#include <Lum/Model/Action.h>
 #include <Lum/Model/Boolean.h>
 
 #include "Sudoku.h"
@@ -34,7 +35,6 @@ private:
   Lum::OS::FontRef       font;
   Lum::OS::ColorRef      presetColor;
   Lum::OS::ColorRef      focusBackgroundColor;
-  Lum::OS::ColorRef      markBackgroundColor;
   Lum::OS::ColorRef      errorColor;
   Lum::OS::ColorRef      correctColor;
   size_t                 boxSize;
@@ -45,6 +45,7 @@ private:
   size_t                 marks[Sudoku::size];
   size_t                 focusedCell;
 
+  Lum::Model::ActionRef  focusChangedAction;
   Lum::Model::BooleanRef finished;
 
 private:
@@ -59,6 +60,8 @@ public:
 
   void GetGame(Game& game) const;
   void GetGame(Sudoku& game) const;
+  void GetRiddle(Sudoku& riddle) const;
+  void GetSolution(Sudoku& solution) const;
 
   void CalcSize();
 
@@ -70,8 +73,13 @@ public:
   bool HandleMouseEvent(const Lum::OS::MouseEvent& event);
   bool HandleKeyEvent(const Lum::OS::KeyEvent& event);
 
+  Lum::Model::Action* GetFocusChangedAction() const;
   Lum::Model::Boolean* GetFinishedModel() const;
 
+  bool IsFocusedCellEditable() const;
+  size_t GetFocusedCellValue() const;
+  void SetFocusedCellValue(size_t value);
+
   void Solve();
 };
 
