@@ -374,7 +374,16 @@ public:
       SaveConfig();
     }
     else if (model==generateAction && generateAction->IsFinished()) {
-      GenerateRiddle();
+      if (Lum::Dlg::Msg::Ask(this,
+                             _(L"GENERATE_TITLE",L"Are you sure?"),
+                             _(L"GENERATE_BODY",
+                               L"Are you sure that you want to generate a new riddle?\n"
+                               L"\n"
+                               L"Generation cannot be undone! The old riddle\n"
+                               L"will be lost!"),
+                             _(L"GENERATE_BUTTONS",L"_Generate!*|_Cancel^"))==0) {
+        GenerateRiddle();
+      }
     }
     else if (model==generateTask.GetFinishedAction() && generateTask.GetFinishedAction()->IsFinished()) {
       solveAction->Enable();
